@@ -8,7 +8,7 @@ frameworks into the ae namespace.
 
 The plan is to integrate the following GUI frameworks until the begin of 2021:
 
-* :mod:`Kivy <ae.kivy_app>`
+* :mod:`Kivy <ae.kivy_app>` - see also :ref:`kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`
 * :mod:`Enaml <ae.enaml_app>`
 * :mod:`Beeware <ae.beeware_app>`
 * :mod:`pyglet <ae.pyglet_app>`
@@ -28,13 +28,13 @@ the :ref:`Kivy Framework <kivy.org>`.
 extended console application environment
 ----------------------------------------
 
-:class:`MainAppBase` inherits directly from the ae namespace class
-:class:`ae console application environment class <ae.console.ConsoleApp>`.
+The abstract base class :class:`MainAppBase` inherits directly from the ae namespace
+class :class:`ae console application environment class <ae.console.ConsoleApp>`.
 The so inherited helper methods are useful for to configure and
 control the run-time of your GUI app via command line arguments,
 :ref:`config-options` and :ref:`config-files`.
 
-.. hint:
+.. hint::
     Please see the documentation of the :mod:`ae.console` namespace
     portion/module for more detailed information.
 
@@ -48,7 +48,7 @@ integrate new gui framework
 
 For to integrate a new Python GUI framework you have to declare a
 new class that inherits from :class:`MainAppBase` and implements at
-least the two abstract methods :meth:`~MainAppBase.on_app_init`
+least the two abstract methods :meth:`~MainAppBase.app_init`
 and :meth:`~MainAppBase.run_app`.
 
 Most GUI frameworks are providing an application class that need to
@@ -56,7 +56,7 @@ be initialized. For to keep a reference to the framework app class
 instance within your main app class you can use the
 :attr:`~MainAppBase.framework_app` attribute of :class:`MainAppBase`.
 
-.. hint:
+.. hint::
     The initialization of :attr:`~MainAppBase.framework_app` is optional.
     Alternatively you could also add the framework application as mixin
     to the main app class.
@@ -67,7 +67,7 @@ look like::
     from new_gui_framework import NewFrameworkApp
 
     class NewFrameworkMainApp(MainAppBase):
-        def on_app_init(self):
+        def app_init(self):
             self.framework_app = NewFrameworkApp()
 
         def run_app(self):
@@ -75,7 +75,7 @@ look like::
 
 Both implementations of these abstract methods will be executed only once
 at app startup. In the first executed method
-:meth:`~MainAppBase.on_app_init` you could initialize
+:meth:`~MainAppBase.app_init` you could initialize
 the GUI framework and prepare it for the app startup. The :meth:`~MainAppClass.run_app`
 method will be called from the main module of your app project for to
 start the framework app instance.
@@ -92,26 +92,36 @@ This namespace portion introduces the section `aeAppState` in the app
 :ref:`config-files`, where any status values can be stored persistently
 for to be recovered on the next startup of your application.
 
-.. hint:
+.. hint::
     The section name is declared by the :data:`APP_STATE_SECTION_NAME`
     constant. If you need to access this config section directly then
     please use this constant instead of the hardcoded section name.
 
 .. _app-state-variables:
 
+This module is providing/pre-defining the following application state variables:
+
+    * :attr:`~MainAppBase.context_id`
+    * :attr:`~MainAppBase.context_path`
+    * :attr:`~MainAppBase.font_size`
+    * :attr:`~MainAppBase.light_theme`
+    * :attr:`~MainAppBase.sound_volume`
+    * :attr:`~MainAppBase.win_rectangle`
+
 Which app state variables are finally used by your app project is (fully data-driven)
 depending on the app state :ref:`config-variables` detected in all the
 :ref:`config-files` that are found/available at run-time of your app.
 The names of the available application state variables can be
 determined with the main app helper method :meth:`~MainAppBase.app_state_keys`.
 
-:class:`MainBaseApp` provides optionally a user-defined font size to your application
-if it detect the app state variable :attr:`~MainAppBase.font_size`. The method
-:meth:`set_font_size` has to be called when the user has changed the font size
-of your app.
+If your application is e.g. supporting a user-defined font size, using the
+provided/pre-defined app state variable :attr:`~MainAppBase.font_size`, then it has to call
+the method :meth:`change_app_state` with the :paramref:`~MainAppBase.change_app_state.state_name`
+set to `font_size` every time when the user has changed the font size of your app.
 
-Another two built-in app state variables are :attr:`~MainAppBase.context_id` and
-:attr:`~MainAppBase.context_path` which will be explained in the next section.
+.. hint::
+    The two built-in app state variables are :attr:`~MainAppBase.context_id` and
+    :attr:`~MainAppBase.context_path` will be explained detailed in the next section.
 
 The :meth:`~MainBaseApp.load_app_states` method is called on instantiation from
 the implemented main app class for to load the values of all
@@ -125,20 +135,38 @@ all app state variables as a dict is determining the method
 :meth:`~MainBaseApp.retrieve_app_states` for you.
 
 Changed app state value that need to be propagated also to the framework
-app instance should be set not via the instance attribute, instead call
-the method :meth:`~MainBaseApp.change_app_state` (which ensures the
-propagation to any duplicated (observable/bound) framework property).
+app instance should never be set via the instance attribute, instead call
+the method :meth:`~MainBaseApp.change_app_state` (which ensures (1) the
+propagation to any duplicated (observable/bound) framework property and
+(2) the event notification to related the main app instance method).
 
 For to to save the app state to the :ref:`config-files` the implementing main app
 instance has to call the method :meth:`~MainBaseApp.save_app_states` - this could be
 done e.g. after the app state has changed or at least on quiting the application.
 
+.. _app-state-constants:
+
+This module is also providing some pre-defined constants that can be optionally
+used in your application in relation to the app states data store and
+for the app state config variables :attr:`~MainAppBase.font_size`
+and :attr:`~MainAppBase.light_theme`:
+
+    * :data:`APP_STATE_SECTION_NAME`
+    * :data:`APP_STATE_VERSION_VAR_NAME`
+    * :data:`APP_STATE_CURRENT_VERSION`
+    * :data:`MIN_FONT_SIZE`
+    * :data:`MAX_FONT_SIZE`
+    * :data:`THEME_LIGHT_BACKGROUND_COLOR`
+    * :data:`THEME_LIGHT_FONT_COLOR`
+    * :data:`THEME_DARK_BACKGROUND_COLOR`
+    * :data:`THEME_DARK_FONT_COLOR`
+
 
 application context
 -------------------
 
 :class:`MainBaseApp` provides your application a integrated context manager,
-which persists application contexts in the :ref:`config-files`.
+which persists application contexts in the :ref:`config-files` as an app state.
 
 A application context is represented by a string that defines e.g.
 the action for to enter into the context, the data that gets currently displayed
@@ -146,30 +174,38 @@ and an index within the data.
 
 The format of this context string/id can be freely defined by your application.
 The app state variable :attr:`~MainAppBase.context_id` stores the current
-context when the app quits for to restore it on the next app start.
+context (or selection) when the app quits for to restore it
+on the next app start.
 
 The context id is initially an empty string. As soon as the user is
-initiating an action your application has to call the
-:meth:`~MainBaseApp.set_context` method for to change the app context.
+changing the current selection your application should call the
+:meth:`~MainBaseApp.change_app_state` passing the string `context_id`
+into the :paramref:`~MainAppBase.change_app_state.state_name` argument
+for to change the app context.
 
 For more complex applications you can specify a path of nested contexts.
 This context path gets represented by the app state variable
 :attr:`~MainAppBase.context_path`, which is a list of context strings/ids.
-For to enter into a deeper/nested context you have to call the
-:meth:`~MainBaseApp.context_enter` method instead of the
-:meth:`~MainBaseApp.set_context` method.
+For to enter into a deeper/nested context you simply call the
+:meth:`~MainBaseApp.context_enter` method instead of calling the
+:meth:`~MainBaseApp.change_app_state` method for the `context_id`
+and `context_path` app states.
 
 
-application events
-------------------
+application state events
+------------------------
 
 The helper method :meth:`~MainAppBase.call_event` can be used to support
 optionally implemented event callback routines.
 
-Internally is this method used for to fire the event `on_context_draw()`
-automatically after a change of the :ref:`application context` or if
-the user changed the font size.
+This method is used internally by this module for to fire notification events when one of
+the app state variables gets changed - like e.g. the :ref:`application context` or if
+the user changes the font size.
 
+The method name of the notification event consists of the prefix ``on_`` followed
+by the variable name of the app state. So on a change of the `font_size` the
+notification event ``on_font_size`` will be called (if exists as a method
+of the main app instance).
 """
 from abc import ABC, abstractmethod
 from configparser import NoSectionError
@@ -182,17 +218,27 @@ from ae.console import ConsoleApp               # type: ignore
 from ae.files import FilesRegister, CachedFile  # type: ignore
 
 
-__version__ = '0.0.5'
+__version__ = '0.0.6'
 
 
-AppStateType = Dict[str, Any]           #: app state config variable type
+AppStateType = Dict[str, Any]                   #: app state config variable type
 
-APP_STATE_SECTION_NAME = 'aeAppState'   #: config section name for to store app state
+APP_STATE_SECTION_NAME = 'aeAppState'           #: config section name for to store app state
+
+#: config variable name for to store the current application state version
 APP_STATE_VERSION_VAR_NAME = 'app_state_version'
-APP_STATE_CURRENT_VERSION = 2
+APP_STATE_CURRENT_VERSION = 2                   #: current application state version
+
+MIN_FONT_SIZE = 15                              #: minimum font size in pixels
+MAX_FONT_SIZE = 99                              #: maximum font size in pixels
+
+THEME_LIGHT_BACKGROUND_COLOR = (1, 1, 1, 1)     #: light theme background color in rgba(0.0 ... 1.0)
+THEME_LIGHT_FONT_COLOR = (0, 0, 0, 1)           #: light theme font color in rgba(0.0 ... 1.0)
+THEME_DARK_BACKGROUND_COLOR = (0, 0, 0, 1)      #: dark theme background color in rgba(0.0 ... 1.0)
+THEME_DARK_FONT_COLOR = (1, 1, 1, 1)            #: dark theme font color in rgba(0.0 ... 1.0)
 
 
-check_all()
+check_all()     # let ae_updater module check/install any outstanding updates or new app versions
 
 
 class MainAppBase(ConsoleApp, ABC):
@@ -202,6 +248,8 @@ class MainAppBase(ConsoleApp, ABC):
     context_path: List[str]                                 #: list of context ids, reflecting recent user actions
     font_size: float = 30.0                                 #: font size used for toolbar and context screens
     light_theme: bool = False                               #: True=light theme/background, False=dark theme
+    sound_volume: float = 0.12                              #: sound volume of current app (0.0=mute, 1.0=max)
+    win_rectangle: tuple = (0, 0, 800, 600)                 #: window coordinates (x, y, width, height)
 
     # generic run-time shortcut references provided by the main app
     framework_app: Any = None                               #: app class instance of the used GUI framework
@@ -226,12 +274,12 @@ class MainAppBase(ConsoleApp, ABC):
         self.load_app_states()
         self.load_images()
         self.load_sounds()
-        self.on_app_init()
+        self.app_init()
 
     # abstract methods
 
     @abstractmethod
-    def on_app_init(self):
+    def app_init(self):
         """ callback to framework api for to initialize an app instance. """
 
     @abstractmethod
@@ -264,14 +312,17 @@ class MainAppBase(ConsoleApp, ABC):
             return event_callback(*args, **kwargs)
         return None
 
-    def change_app_state(self, state_name: str, new_value: Any):
+    def change_app_state(self, state_name: str, new_value: Any, send_event: bool = True):
         """ change single app state item to value in self.attribute and app_state dict item.
 
         :param state_name:  name of the app state to change.
         :param new_value:   new value of the app state to change.
+        :param send_event:  pass False to prevent to send/call the on_<state_name> event to the main app instance.
         """
         setattr(self, state_name, new_value)
         self._change_observable(state_name, new_value)
+        if send_event:
+            self.call_event('on_' + state_name)
 
     def _change_observable(self, name: str, value: Any):
         """ change observable attribute/member/property in framework_app instance.
@@ -292,7 +343,8 @@ class MainAppBase(ConsoleApp, ABC):
         :param next_context_id: context id within the entering/new child context.
         """
         self.context_path.append(context_id)
-        self.set_context(next_context_id)
+        self.change_app_state('context_path', self.context_path)
+        self.change_app_state('context_id', next_context_id)
 
     def context_leave(self, next_context_id: str = ''):
         """ user navigates up in the data tree.
@@ -300,14 +352,15 @@ class MainAppBase(ConsoleApp, ABC):
         :param next_context_id: context id to set in the parent context.
         """
         list_name = self.context_path.pop()
-        self.set_context(next_context_id or list_name)
+        self.change_app_state('context_path', self.context_path)
+        self.change_app_state('context_id', next_context_id or list_name)
 
     def find_image_file(self, image_name: str, height: float = 32.0, light_theme: bool = True) -> str:
         """ find best fitting image in img app folder.
 
         :param image_name:      name of the image (file name without extension).
         :param height:          preferred height of the image/icon.
-        :param light_theme:     preferred thema (dark/light) of the image.
+        :param light_theme:     preferred theme (dark/light) of the image.
         :return:                image file path if found else empty string.
         """
         def property_matcher(file):
@@ -404,26 +457,6 @@ class MainAppBase(ConsoleApp, ABC):
         self.load_cfg_files()
         return err_msg
 
-    def set_context(self, context_id: str, redraw: bool = True):
-        """ propagate change of context path and context/current id/item and display changed context.
-
-        :param context_id:  name of new current item.
-        :param redraw:      pass False to prevent to redraw the context screens.
-        """
-        self.dpo(f"set_context({context_id})")
-        self.change_app_state('context_path', self.context_path)
-        self.change_app_state('context_id', context_id)
-        if redraw:
-            self.call_event('on_context_draw')
-
-    def set_font_size(self, font_size: float):
-        """ change font size.
-
-        :param font_size:   font size in pixels.
-        """
-        self.change_app_state('font_size', font_size)
-        self.call_event('on_context_draw')
-
     def setup_app_states(self, app_state: AppStateType):
         """ put app state variables into main app instance for to prepare framework app.run_app.
 
